"""
API Client for https://lolz.live & https://lzt.market & https://antipublic.one
https://pypi.org/project/LOLZTEAM/
https://github.com/AS7RIDENIED/LOLZTEAM
"""
from LOLZTEAM.Client import Forum, Market, Antipublic

__all__ = ["Forum", "Market", "Antipublic"]
